/***********************************************************
 *
 *	boot.c
 *
 *  This is vector and interrupt program with C.
 *
 * history : 2008/04/01	start
 * 			 2008/06/04	modify vector table
 *
 *	COPYRIGHT (C) 2015, SEIKO EPSON CORP. All Rights Reserved
 ***********************************************************/

/* #include */
#include <string.h>

/* Prototype */
void boot(void);
void clearBss(void);
void copyLmaToVma(void);
void end(void);
void dummy(void)__attribute__ ((interrupt_handler));
void nmi(void)__attribute__ ((interrupt_handler));
extern void emu_copro_process(void);
/* extern */
extern int main(void);
extern void emu_copro_process(void);
extern unsigned long __START_bss;
extern unsigned long __END_bss;
extern unsigned long __START_data;
extern unsigned long __START_data_lma;
extern unsigned long __END_data;


/* special definitions for vector table  */
typedef void func(void);
#ifdef __POINTER16
#define VECTOR(vec)  vec,0
#else
#define VECTOR(vec)  vec
#endif

/* vector table */
func *const vector[] = {		// No	Base+	Name
	VECTOR(boot),				// 0	00		Reset
	VECTOR(dummy),				// 1	04		Illegal Adress Int.
	VECTOR(nmi),				// 2	08		NMI
	VECTOR(emu_copro_process),	// 3	0c		Copro
	VECTOR(dummy),				// 4	10
	VECTOR(dummy),				// 5	14
	VECTOR(dummy),				// 6	18
	VECTOR(dummy),				// 7	1c
	VECTOR(dummy),				// 8	20
	VECTOR(dummy),				// 9	24
	VECTOR(dummy),				// 10	28
	VECTOR(dummy),				// 11	2c
	VECTOR(dummy),				// 12	30
	VECTOR(dummy),				// 13	34
	VECTOR(dummy),				// 14	38
	VECTOR(dummy),				// 15	3c
	VECTOR(dummy),				// 16	40
	VECTOR(dummy),				// 17	44
	VECTOR(dummy),				// 18	48
	VECTOR(dummy),				// 19	4c
	VECTOR(dummy),				// 20	50
	VECTOR(dummy),				// 21	54
	VECTOR(dummy),				// 22	58
	VECTOR(dummy),				// 23	5c
	VECTOR(dummy),				// 24	60
	VECTOR(dummy),				// 25	64
	VECTOR(dummy),				// 26	68
	VECTOR(dummy),				// 27	6c
	VECTOR(dummy),				// 28	70
	VECTOR(dummy),				// 29	74
	VECTOR(dummy),				// 30	78
	VECTOR(dummy),				// 31	7C
};


/*
 * Boot function.
 * First, it resets the Stack Pointer, clear the bss area, and copy LMA to VMA.
 * Then, it calls "main" function and calls "end" function.
 *
 * return: void
 * history: 2011/12/01	start
 */
void boot(void) {
	asm("xld.a  %sp, 0x7c0");  		// Set SP in RAM

	clearBss();						// clear the bss area
	copyLmaToVma();					// copy LMA data to VMA data

	asm("ei");						//interrupt enable

	main(); 						// Call main
	end(); 							// Call end
}


/*
 * ClearBss function.
 * This function clears the bss area.
 *
 * return: void
 * history: 2007/05/08	start
 */
void clearBss(void) {
	unsigned long ulCnt;
	unsigned char *pcData;

	pcData = (unsigned char *)&__START_bss;
	for( ulCnt = 0; ulCnt < ((unsigned char *)&__END_bss - (unsigned char *)&__START_bss); ++ulCnt ) {
		*pcData = 0;
		++pcData;
	}
}


/*
 * copyLmaToVma function.
 * This function copys LMA data to VMA data.
 *
 * return: void
 * history: 2012/12/01	start
 */
void copyLmaToVma(void) {
	memcpy((unsigned char *)&__START_data, (unsigned char *)&__START_data_lma, ((unsigned char *)&__END_data - (unsigned char *)&__START_data));
}


/*
 * End function.
 * This function has only infinite loop.
 *
 * return: void
 * history: 2007/05/07	start
 */
void end(void) {
	// infinite loop
	while(1) {
	}
}


/*
 * Non Maskable Interrupt function.
 *
 * return: void
 * history: 2007/05/07	start
 */
void nmi(void) {
	// Processing is undecided.
}


/*
 * Dummy Interrupt function.
 * This function has only infinite loop.
 *
 * return: void
 * history: 2007/05/07	start
 */
void dummy(void) {
	// infinite loop
	while(1) {
	}
}
